<?php
/* --------------------------------------------------------------
 AdminAction.php 2020-09-15
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Admin\Application\Http;

use Gambio\Admin\Layout\Renderer\GambioAdminRenderer;
use Gambio\Core\Application\Modules\ModuleAction;
use Gambio\Core\Language\TextManager;
use Gambio\Core\TemplateEngine\Exceptions\RenderingFailedException;

/**
 * Class AdminAction
 * @package Gambio\Admin\Application\Http
 */
abstract class AdminAction extends ModuleAction
{
    /**
     * @var GambioAdminRenderer
     */
    private $renderer;
    
    /**
     * @var TextManager
     */
    private $textManager;
    
    /**
     * @var bool
     */
    private $isLargeLayout = false;
    
    
    /**
     * Admin action initialization.
     *
     * @param GambioAdminRenderer $renderer
     * @param TextManager         $textManager
     */
    public function initAdminAction(
        GambioAdminRenderer $renderer,
        TextManager $textManager
    ): void {
        $this->renderer    = $renderer;
        $this->textManager = $textManager;
    }
    
    
    /**
     * Renders the template.
     *
     * @param string $pageTitle
     * @param string $templatePath
     * @param array  $data
     *
     * @return string
     * @throws RenderingFailedException
     */
    public function render(string $pageTitle, string $templatePath, array $data = []): string
    {
        if ($this->isLargeLayout) {
            $data['isLargeLayout'] = true;
        }
        $data['pageTitle'] = $pageTitle;
        
        return $this->renderer->render($templatePath, $data);
    }
    
    
    /**
     * Replaces the renderer.
     *
     * This method can be inflected on child classes to provide a differently function renderer.
     *
     * @param GambioAdminRenderer $renderer
     */
    public function replaceRenderer(GambioAdminRenderer $renderer): void
    {
        $this->renderer = $renderer;
    }
    
    
    /**
     * Translates the section phrase.
     *
     * @param string   $phrase
     * @param string   $section
     * @param int|null $languageId
     *
     * @return string
     */
    protected function translate(string $phrase, string $section, int $languageId = null): string
    {
        return $this->textManager->getPhraseText($phrase, $section, $languageId);
    }
    
    
    /**
     * Enables the large page layout.
     *
     * Technically, this method sets an internal flag, so the layout hides
     * the message stack and content navigation.
     */
    protected function enableLargePageLayout(): void
    {
        $this->isLargeLayout = true;
    }
}