<?php
/* --------------------------------------------------------------
 ModuleRegistryRepository.php 2020-09-17
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Admin\Application\Registry\ModuleRegistry\App\Data;

use Gambio\Admin\Application\Registry\ModuleRegistry\Model\ModuleRegistry;
use Gambio\Admin\Application\Registry\ModuleRegistry\Model\ValueObject\ModuleRegistry as Registry;
use Gambio\Admin\Application\Registry\ModuleRegistry\Services\ModuleRegistryRepository as RegistryRepository;
use Gambio\Core\Application\ValueObjects\Path;
use RecursiveDirectoryIterator;
use RecursiveIteratorIterator;
use SplFileInfo;

/**
 * Class ModuleRegistryRepository
 * @package Gambio\Admin\Application\Registry\ModuleRegistry\App\Data
 * @codeCoverageIgnore
 */
class ModuleRegistryRepository implements RegistryRepository
{
    /**
     * @var GxModulesPaths
     */
    private $gxModulesPaths;
    
    /**
     * @var Path
     */
    private $path;
    
    
    /**
     * ModuleRegistryRepository constructor.
     *
     * @param GxModulesPaths $gxModulesPaths
     * @param Path           $path
     */
    public function __construct(GxModulesPaths $gxModulesPaths, Path $path)
    {
        $this->gxModulesPaths = $gxModulesPaths;
        $this->path           = $path;
    }
    
    
    /**
     * @inheritDoc
     */
    public function getRegistry(): ModuleRegistry
    {
        $registryData = $this->searchRegistryData();
        
        return new Registry(
            $registryData['modules'], $registryData['serviceProviders'], $registryData['autoloaderPaths']
        );
    }
    
    
    /**
     * Searches in the GXModules directory for module registry data.
     *
     * @return array[]
     */
    private function searchRegistryData(): array
    {
        $modules          = [];
        $serviceProviders = [];
        $autoloaderPaths  = [];
        
        foreach ($this->gxModulesPaths->modules() as $module) {
            foreach ($this->recursiveDirectoryIterator($module) as $moduleFile) {
                $filename = $moduleFile->getFilename();
                $path     = $moduleFile->getPathname();
                
                if ($this->endsWith('Module.php', $filename)) {
                    $modules[] = $this->pathToFqn($path);
                    continue;
                }
                if ($this->endsWith('ServiceProvider.php', $filename)) {
                    $serviceProviders[] = $this->pathToFqn($path);
                    continue;
                }
                if ($this->endsWith('autoload.php', $filename)) {
                    $autoloaderPaths[] = $path;
                }
            }
        }
        
        return [
            'modules'          => $modules,
            'serviceProviders' => $serviceProviders,
            'autoloaderPaths'  => $autoloaderPaths,
        ];
    }
    
    
    /**
     * Translates the path to a GXModules namespace.
     *
     * @param string $path
     *
     * @return string
     */
    private function pathToFqn(string $path): string
    {
        return str_replace(["{$this->path->base()}/", '.php', '/'], ['', '', '\\'], $path);
    }
    
    
    /**
     * Checks if subject ends with expected string.
     *
     * @param string $searchFor
     * @param string $subject
     *
     * @return bool
     */
    private function endsWith(string $searchFor, string $subject): bool
    {
        $length = strlen($searchFor);
        if ($length === 0) {
            return true;
        }
        
        return (substr($subject, -$length) === $searchFor);
    }
    
    
    /**
     * Utility method to create a recursive directory iterator.
     *
     * @param string $path
     *
     * @return RecursiveIteratorIterator|SplFileInfo[]
     */
    private function recursiveDirectoryIterator(string $path): RecursiveIteratorIterator
    {
        return new RecursiveIteratorIterator(
            new RecursiveDirectoryIterator(
                $path, RecursiveDirectoryIterator::SKIP_DOTS
            ), RecursiveIteratorIterator::LEAVES_ONLY
        );
    }
}