<?php
/*------------------------------------------------------------------------------
  ActionIndex.php 2021-06-21
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2021 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  -----------------------------------------------------------------------------*/
declare(strict_types=1);

namespace Gambio\Admin\Modules\Dashboard\App\Actions;

use Curl\Curl;
use Gambio\Admin\Application\Http\VuePageAction;
use Gambio\Admin\Modules\Dashboard\App\Data\DashboardConfigurationStorage;
use Gambio\Admin\Modules\Dashboard\Factories\ShopOriginFactory;
use Gambio\Admin\Modules\Dashboard\Html\HtmlProvider;
use Gambio\Admin\Modules\Dashboard\ValueObjects\DashboardData;
use Gambio\Admin\Modules\SetupWizard\SetupWizardServiceInterface;
use Gambio\Admin\Modules\Statistics\StatisticsServiceInterface;
use Gambio\Core\Application\Modules\Http\Response as HttpResponse;
use Gambio\Core\Application\ValueObjects\Url;
use Gambio\Core\Configuration\ConfigurationService;

/**
 * Class ActionIndex
 * @package Gambio\Admin\Modules\Dashboard\App\Actions
 * @codeCoverageIgnore
 */
class ActionIndex extends VuePageAction
{
    /**
     * @var StatisticsServiceInterface
     */
    private $statisticService;
    /**
     * @var SetupWizardServiceInterface
     */
    private $setupWizardService;
    /**
     * @var DashboardConfigurationStorage
     */
    private $storage;
    /**
     * @var HtmlProvider
     */
    private $htmlProvider;
    /**
     * @var ConfigurationService
     */
    private $configurationService;
    /**
     * @var Curl
     */
    private $curl;
    
    
    /**
     * ActionIndex constructor.
     *
     * @param StatisticsServiceInterface    $statisticService
     * @param SetupWizardServiceInterface   $setupWizardService
     * @param DashboardConfigurationStorage $storage
     * @param HtmlProvider                  $htmlProvider
     * @param Url                           $url
     * @param ConfigurationService          $configurationService
     * @param Curl                          $curl
     */
    public function __construct(
        StatisticsServiceInterface $statisticService,
        SetupWizardServiceInterface $setupWizardService,
        DashboardConfigurationStorage $storage,
        HtmlProvider $htmlProvider,
        Url $url,
        ConfigurationService $configurationService,
        Curl $curl
    ) {
        $this->statisticService     = $statisticService;
        $this->setupWizardService   = $setupWizardService;
        $this->storage              = $storage;
        $this->htmlProvider         = $htmlProvider;
        $this->url                  = $url;
        $this->configurationService = $configurationService;
        $this->curl                 = $curl;
    }
    
    
    protected function handle(): HttpResponse
    {
        $shopKeyConfig = $this->configurationService->find('configuration/GAMBIO_SHOP_KEY');
        $shopKey       = $shopKeyConfig ? $shopKeyConfig->value() : '';
        
        $shopUrl = $this->url->base();
        
        $shopVersionConfig = $this->configurationService->find('gm_configuration/INSTALLED_VERSION');
        $shopVersion       = $shopVersionConfig ? $shopVersionConfig->value() : '';
        
        $statistics            = $this->statisticService->statistics();
        $steps                 = $this->setupWizardService->getSteps();
        $userPreferredCategory = $this->statisticService->getUserPreferredCategory();
        $userPreferredPeriod   = $this->statisticService->getUserPreferredPeriod();
        $data                  = new DashboardData($shopKey,
                                                   $shopUrl,
                                                   $shopVersion,
                                                   $statistics,
                                                   $steps,
                                                   $this->storage->isExternalSocialMediaEmbedsAllowed(),
                                                   $userPreferredCategory,
                                                   $userPreferredPeriod,
                                                   (new ShopOriginFactory)->createShopOrigin());
        $json                  = json_encode($data, JSON_PRETTY_PRINT);
        $html                  = $this->htmlProvider->dashboardHtml();
        
        if (strpos($html, '[[dashboard_placeholder]]') === false) {
            $html .= "<script id='dashboardStatsJson' type='application/json'>$json</script>";
        } else {
            
            $html = str_replace('[[dashboard_placeholder]]', $json, $html);
            /**
             * The next replaces are here to fix some problems with the HTML template
             */
            $search  = [
                'https://www.gambio.de/files/admin-news/js',
                'https://www.gambio.com/files/admin-news/js'
            ];
            $replace = 'dashboard/javascript';
            
            $html    = str_replace($search, $replace, $html);
            $search  = 'title[language]';
            $replace = 'title';
            $html    = str_replace($search, $replace, $html);
        }
        
        $this->enableLargePageLayout();
        $title    = $this->translate('PAGE_TITLE', 'dashboard');
        $template = dirname(__DIR__, 2) . '/ui/dashboard.html';
        
        return $this->response->write($this->render($title, $template, ['content' => $html]));
    }
    
    
    /**
     * @inheritDoc
     */
    protected function jsEntrypoint(): string
    {
        return 'dashboard';
    }
}