<?php
/* --------------------------------------------------------------
   ParcelServiceCriteria.php 2020-08-28
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\ParcelService\App\Data\Filter;

use Doctrine\DBAL\Query\QueryBuilder;
use Gambio\Admin\Modules\ParcelService\Model\ValueObjects\ParcelServiceFilters as ParcelServiceFiltersInterface;
use Webmozart\Assert\Assert;

/**
 * Class ParcelServiceRestApiCriteria
 *
 * @package Gambio\Admin\Modules\ParcelService\SqlCriteria
 */
class ParcelServiceFilters implements ParcelServiceFiltersInterface
{
    /**
     * @var array<string, array<string, string>>
     */
    private $filters = [];
    
    /**
     * @var string[]
     */
    private static $attributeColumnMapping = [
        'id'                        => 'parcel_services.parcel_service_id',
        'name'                      => 'parcel_services.name',
        'isDefault'                 => 'parcel_services.default',
        'descriptions.languageCode' => 'languages.code',
        'descriptions.url'          => 'parcel_services_description.url',
        'descriptions.comment'      => 'parcel_services_description.comment',
    ];
    
    /**
     * @var string[]
     */
    private static $allowedForLikeOperation = [
        'name',
        'descriptions.languageCode',
        'descriptions.url',
        'descriptions.comment',
    ];
    
    /**
     * @var string[]
     */
    private static $allowedForNumericOperation = [
        'id',
    ];
    
    
    /**
     * ParcelServiceFilters constructor.
     */
    private function __construct()
    {
    }
    
    
    public static function create(array $filters): ParcelServiceFilters
    {
        Assert::isMap($filters, 'Provided filters need to be a map, mapping document attribute to filter.');
        Assert::allKeyExists($filters, 'value', 'Provided filters must contain a "value" key.');
        Assert::allKeyExists($filters, 'operation', 'Provided filters must contain a "operation" key.');
        
        $self = new self();
        foreach ($filters as $attribute => $filter) {
            $self->addFilter($attribute, $filter);
        }
        
        return $self;
    }
    
    
    /**
     * @param QueryBuilder $query
     */
    public function applyToQuery(QueryBuilder $query): void
    {
        foreach ($this->filters as $column => $filter) {
            $value     = $filter['value'];
            $operation = $filter['operation'];
            $parameter = ':criteria_filter_' . str_replace('.', '_', $column);
            
            if ($operation === 'like') {
                $value      = str_replace(['%', '*'], ['\%', '%'], $value);
                $filterExpr = $query->expr()->like($column, $parameter);
            } elseif ($operation === 'gt') {
                $filterExpr = $query->expr()->gt($column, $parameter);
            } elseif ($operation === 'gte') {
                $filterExpr = $query->expr()->gte($column, $parameter);
            } elseif ($operation === 'lt') {
                $filterExpr = $query->expr()->lt($column, $parameter);
            } elseif ($operation === 'lte') {
                $filterExpr = $query->expr()->lte($column, $parameter);
            } else {
                $filterExpr = $query->expr()->eq($column, $parameter);
            }
            
            $query->andWhere($filterExpr)->setParameter($parameter, $value);
        }
    }
    
    
    /**
     * @param string $attribute
     * @param array  $filter
     */
    private function addFilter(string $attribute, array $filter): void
    {
        if (!array_key_exists($attribute, self::$attributeColumnMapping)) {
            return;
        }
        
        $operations = ['like', 'eq', 'gt', 'gte', 'lt', 'lte'];
        Assert::oneOf($filter['operation'],
                      $operations,
                      'Provided operation needs to be one of: ' . implode(', ', $operations) . '; Got: '
                      . $filter['operation']);
        
        if ($filter['operation'] === 'like') {
            Assert::oneOf($attribute,
                          self::$allowedForLikeOperation,
                          'Provided attribute is not allowed for "like" operation.');
        } elseif (in_array($filter['operation'], ['gt', 'gte', 'lt', 'lte'])) {
            Assert::oneOf($attribute,
                          self::$allowedForNumericOperation,
                          'Provided attribute is not allowed for a numeric operation.');
        }
        
        $this->filters[self::$attributeColumnMapping[$attribute]] = $filter;
    }
}