<?php
/* --------------------------------------------------------------
   ParcelServiceRepository.php 2020-08-28
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2020 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\ParcelService\App\Data;

use Gambio\Admin\Modules\ParcelService\Model\Collections\ParcelServiceDescriptions;
use Gambio\Admin\Modules\ParcelService\Model\Collections\ParcelServiceIds;
use Gambio\Admin\Modules\ParcelService\Model\Collections\ParcelServices;
use Gambio\Admin\Modules\ParcelService\Model\Events\CreatedParcelService;
use Gambio\Admin\Modules\ParcelService\Model\Events\DeletedParcelService;
use Gambio\Admin\Modules\ParcelService\Model\ParcelService;
use Gambio\Admin\Modules\ParcelService\Model\ValueObjects\ParcelServiceFilters;
use Gambio\Admin\Modules\ParcelService\Model\ValueObjects\ParcelServiceId;
use Gambio\Admin\Modules\ParcelService\Model\ValueObjects\ParcelServicePagination;
use Gambio\Admin\Modules\ParcelService\Model\ValueObjects\ParcelServiceSorting;
use Gambio\Admin\Modules\ParcelService\Services\ParcelServiceRepository as ParcelServiceRepositoryInterface;
use Gambio\Core\Event\Abstracts\AbstractEventDispatchingRepository;
use InvalidArgumentException;
use Psr\EventDispatcher\EventDispatcherInterface;
use Webmozart\Assert\Assert;

/**
 * Class ParcelServiceRepository
 *
 * @package Gambio\Admin\Modules\ParcelService\App\Data
 */
class ParcelServiceRepository extends AbstractEventDispatchingRepository implements ParcelServiceRepositoryInterface
{
    /**
     * @var ParcelServiceMapper
     */
    private $mapper;
    
    /**
     * @var ParcelServiceReader
     */
    private $reader;
    
    /**
     * @var ParcelServiceWriter
     */
    private $writer;
    
    
    /**
     * ParcelServiceRepository constructor.
     *
     * @param ParcelServiceMapper      $mapper
     * @param ParcelServiceReader      $reader
     * @param ParcelServiceWriter      $writer
     * @param EventDispatcherInterface $eventDispatcher
     */
    public function __construct(
        ParcelServiceMapper $mapper,
        ParcelServiceReader $reader,
        ParcelServiceWriter $writer,
        EventDispatcherInterface $eventDispatcher
    ) {
        $this->mapper = $mapper;
        $this->reader = $reader;
        $this->writer = $writer;
        
        $this->setEventDispatcher($eventDispatcher);
    }
    
    
    /**
     * @inheritDoc
     */
    public function filterParcelServices(
        ParcelServiceFilters $filters,
        ParcelServiceSorting $sorting,
        ParcelServicePagination $pagination
    ): ParcelServices {
        $parcelServicesData = $this->reader->getFilteredParcelServicesData($filters, $sorting, $pagination);
        
        return $this->mapper->mapParcelServices($parcelServicesData);
    }
    
    
    /**
     * @inheritDoc
     */
    public function getParcelServicesTotalCount(ParcelServiceFilters $criteria): int
    {
        return $this->reader->getParcelServicesTotalCount($criteria);
    }
    
    
    /**
     * @inheritDoc
     */
    public function getAllParcelServices(): ParcelServices
    {
        $parcelServicesData = $this->reader->getAllParcelServicesData();
        
        return $this->mapper->mapParcelServices($parcelServicesData);
    }
    
    
    /**
     * @inheritDoc
     */
    public function getParcelServiceById(ParcelServiceId $id): ParcelService
    {
        $parcelServiceData = $this->reader->getParcelServiceDataById($id);
        
        return $this->mapper->mapParcelService($parcelServiceData);
    }
    
    
    /**
     * @inheritDoc
     */
    public function createParcelService(
        string $name,
        ParcelServiceDescriptions $descriptions,
        bool $isDefault = false
    ): ParcelServiceId {
        $idValue = $this->writer->createParcelService($name, $descriptions, $isDefault);
        $id      = $this->mapper->mapParcelServiceId($idValue);
        
        $this->dispatchEvent(CreatedParcelService::create($id));
        
        return $id;
    }
    
    
    /**
     * @inheritDoc
     */
    public function createMultipleParcelServices(
        array $names,
        array $descriptions,
        array $isDefaultFlags = []
    ): ParcelServiceIds {
        Assert::allIsInstanceOf($descriptions,
                                ParcelServiceDescriptions::class,
                                'All descriptions need to implement the "' . ParcelServiceDescriptions::class
                                . '" interface.');
        if (count($names) !== count($descriptions)) {
            throw new InvalidArgumentException('The number of given names must match the number of descriptions.');
        }
        
        $idValues = $this->writer->createMultipleParcelServices($names, $descriptions, $isDefaultFlags);
        
        $ids = $this->mapper->mapParcelServiceIds(...$idValues);
        foreach ($ids as $id) {
            $this->dispatchEvent(CreatedParcelService::create($id));
        }
        
        return $ids;
    }
    
    
    /**
     * @inheritDoc
     */
    public function storeParcelServices(ParcelService ...$parcelServices): void
    {
        $this->writer->updateParcelServices(...$parcelServices);
        foreach ($parcelServices as $parcelService) {
            $this->dispatchEntityEvents($parcelService);
        }
    }
    
    
    /**
     * @inheritDoc
     */
    public function deleteParcelServices(ParcelServiceId ...$ids): void
    {
        $this->writer->deleteParcelServices(...$ids);
        foreach ($ids as $id) {
            $this->dispatchEvent(DeletedParcelService::create($id));
        }
    }
}