<?php
/* --------------------------------------------------------------
   ParcelServiceWriter.php 2020-09-01
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2020 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\ParcelService\App\Data;

use Doctrine\DBAL\Connection;
use Exception;
use Gambio\Admin\Modules\ParcelService\Model\Collections\ParcelServiceDescriptions;
use Gambio\Admin\Modules\ParcelService\Model\Exceptions\CreationOfParcelServicesFailed;
use Gambio\Admin\Modules\ParcelService\Model\Exceptions\DeletionOfParcelServicesFailed;
use Gambio\Admin\Modules\ParcelService\Model\Exceptions\StorageOfParcelServicesFailed;
use Gambio\Admin\Modules\ParcelService\Model\ParcelService;
use Gambio\Admin\Modules\ParcelService\Model\ValueObjects\ParcelServiceId;
use Gambio\Core\Language\Language;
use Gambio\Core\Language\LanguageService;

/**
 * Class ParcelServiceWriter
 *
 * @package Gambio\Admin\Modules\ParcelService\App\Data
 */
class ParcelServiceWriter
{
    /**
     * @var Connection
     */
    private $db;
    
    /**
     * @var LanguageService
     */
    private $languageService;
    
    
    /**
     * ParcelServiceWriter constructor.
     *
     * @param Connection $db
     */
    public function __construct(Connection $db, LanguageService $languageService)
    {
        $this->db              = $db;
        $this->languageService = $languageService;
    }
    
    
    /**
     * @param string                    $name
     * @param ParcelServiceDescriptions $descriptions
     * @param bool                      $isDefault
     *
     * @return int
     *
     * @throws CreationOfParcelServicesFailed
     */
    public function createParcelService(
        string $name,
        ParcelServiceDescriptions $descriptions,
        bool $isDefault = false
    ): int {
        try {
            if ($isDefault) {
                $this->db->createQueryBuilder()->update('parcel_services')->set('`default`', '0')->execute();
            }
            
            $this->db->createQueryBuilder()
                ->insert('parcel_services')
                ->setValue('`name`', ':name')
                ->setValue('`default`', ':default')
                ->setParameter('name', $name)
                ->setParameter('default', (int)$isDefault)
                ->execute();
            $id = $this->db->lastInsertId();
            
            /** @var Language $language */
            foreach ($this->languageService->getAvailableLanguages() as $language) {
                $this->db->createQueryBuilder()
                    ->insert('parcel_services_description')
                    ->setValue('parcel_service_id', ':parcel_service_id')
                    ->setValue('language_id', ':language_id')
                    ->setValue('url', ':url')
                    ->setValue('comment', ':comment')
                    ->setParameter('parcel_service_id', $id)
                    ->setParameter('language_id', $language->id())
                    ->setParameter('url', $descriptions->url($language->code()))
                    ->setParameter('comment', $descriptions->comment($language->code()))
                    ->execute();
            }
            
            return (int)$id;
        } catch (Exception $exception) {
            throw CreationOfParcelServicesFailed::becauseOfException($exception);
        }
    }
    
    
    /**
     * @param string[]                    $names
     * @param ParcelServiceDescriptions[] $descriptions
     * @param bool[]                      $isDefaultFlags
     *
     * @return int[]
     *
     * @throws CreationOfParcelServicesFailed
     */
    public function createMultipleParcelServices(array $names, array $descriptions, array $isDefaultFlags = []): array
    {
        try {
            $this->db->beginTransaction();
            $ids = [];
            foreach ($names as $index => $name) {
                $ids[] = $this->createParcelService($name, $descriptions[$index], $isDefaultFlags[$index] ?? false);
            }
            $this->db->commit();
        } catch (CreationOfParcelServicesFailed $exception) {
            $this->db->rollBack();
            throw $exception;
        } catch (Exception $exception) {
            $this->db->rollBack();
            throw CreationOfParcelServicesFailed::becauseOfException($exception);
        }
        
        return $ids;
    }
    
    
    /**
     * @param ParcelService $parcelService
     */
    private function updateParcelService(ParcelService $parcelService): void
    {
        if ($parcelService->isDefault()) {
            $this->db->createQueryBuilder()->update('parcel_services')->set('`default`', '0')->execute();
        }
        
        $this->db->createQueryBuilder()
            ->update('parcel_services')
            ->set('`name`', ':name')
            ->set('`default`', ':default')
            ->where('parcel_service_id = :id')
            ->setParameter('name', $parcelService->name())
            ->setParameter('default', (int)$parcelService->isDefault())
            ->setParameter('id', $parcelService->id())
            ->execute();
        
        $this->db->createQueryBuilder()
            ->delete('parcel_services_description')
            ->where('parcel_service_id = :id')
            ->setParameter('id', $parcelService->id())
            ->execute();
        
        /** @var Language $language */
        foreach ($this->languageService->getAvailableLanguages() as $language) {
            $this->db->createQueryBuilder()
                ->insert('parcel_services_description')
                ->setValue('parcel_service_id', ':id')
                ->setValue('language_id', ':language_id')
                ->setValue('url', ':url')
                ->setValue('comment', ':comment')
                ->setParameter('url', $parcelService->url($language->code()))
                ->setParameter('comment', $parcelService->comment($language->code()))
                ->setParameter('id', $parcelService->id())
                ->setParameter('language_id', $language->id())
                ->execute();
        }
    }
    
    
    /**
     * @param ParcelService ...$parcelServices
     *
     * @throws StorageOfParcelServicesFailed
     */
    public function updateParcelServices(ParcelService ...$parcelServices): void
    {
        try {
            $this->db->beginTransaction();
            foreach ($parcelServices as $parcelService) {
                $this->updateParcelService($parcelService);
            }
            $this->db->commit();
        } catch (Exception $exception) {
            $this->db->rollBack();
            throw StorageOfParcelServicesFailed::becauseOfException($exception);
        }
    }
    
    
    /**
     * @param ParcelServiceId $id
     */
    private function deleteParcelService(ParcelServiceId $id): void
    {
        $this->db->createQueryBuilder()
            ->delete('parcel_services')
            ->where('parcel_service_id = :id')
            ->setParameter('id', $id->value())
            ->execute();
        
        $this->db->createQueryBuilder()
            ->delete('parcel_services_description')
            ->where('parcel_service_id = :id')
            ->setParameter('id', $id->value())
            ->execute();
    }
    
    
    /**
     * @param ParcelServiceId ...$ids
     *
     * @throws DeletionOfParcelServicesFailed
     */
    public function deleteParcelServices(ParcelServiceId ...$ids): void
    {
        try {
            $this->db->beginTransaction();
            foreach ($ids as $id) {
                $this->deleteParcelService($id);
            }
            $this->db->commit();
        } catch (Exception $exception) {
            $this->db->rollBack();
            throw DeletionOfParcelServicesFailed::becauseOfException($exception);
        }
    }
}