<?php
/* --------------------------------------------------------------
   ParcelService.php 2020-08-28
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2020 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\ParcelService\Model;

use Gambio\Admin\Modules\ParcelService\Model\Collections\ParcelServiceDescriptions;
use Gambio\Admin\Modules\ParcelService\Model\Events\SetParcelServiceAsDefault;
use Gambio\Admin\Modules\ParcelService\Model\Events\UpdatedParcelServiceDescriptions;
use Gambio\Admin\Modules\ParcelService\Model\Events\UpdatedParcelServiceName;
use Gambio\Admin\Modules\ParcelService\Model\ValueObjects\ParcelServiceId;
use Gambio\Core\Event\Abstracts\AbstractEventRaisingEntity;

/**
 * Class ParcelService
 *
 * @package Gambio\Admin\Modules\ParcelService\Model
 */
class ParcelService extends AbstractEventRaisingEntity
{
    /**
     * @var ParcelServiceId
     */
    private $id;
    
    /**
     * @var string
     */
    private $name;
    
    /**
     * @var bool
     */
    private $isDefault;
    
    /**
     * @var ParcelServiceDescriptions
     */
    private $descriptions;
    
    
    /**
     * ParcelService constructor.
     *
     * @param ParcelServiceId           $id
     * @param string                    $name
     * @param bool                      $isDefault
     * @param ParcelServiceDescriptions $descriptions
     */
    private function __construct(
        ParcelServiceId $id,
        string $name,
        bool $isDefault,
        ParcelServiceDescriptions $descriptions
    ) {
        $this->id           = $id;
        $this->name         = $name;
        $this->isDefault    = $isDefault;
        $this->descriptions = $descriptions;
    }
    
    
    /**
     * @param ParcelServiceId           $id
     * @param string                    $name
     * @param bool                      $isDefault
     * @param ParcelServiceDescriptions $descriptions
     *
     * @return ParcelService
     */
    public static function create(
        ParcelServiceId $id,
        string $name,
        bool $isDefault,
        ParcelServiceDescriptions $descriptions
    ): ParcelService {
        return new self($id, $name, $isDefault, $descriptions);
    }
    
    
    /**
     * @return int
     */
    public function id(): int
    {
        return $this->id->value();
    }
    
    
    /**
     * @return string
     */
    public function name(): string
    {
        return $this->name;
    }
    
    
    /**
     * @return bool
     */
    public function isDefault(): bool
    {
        return $this->isDefault;
    }
    
    
    /**
     * @param string $languageCode
     *
     * @return string
     */
    public function url(string $languageCode): string
    {
        return $this->descriptions->url($languageCode);
    }
    
    
    /**
     * @param string $languageCode
     *
     * @return string
     */
    public function comment(string $languageCode): string
    {
        return $this->descriptions->comment($languageCode);
    }
    
    
    /**
     * @param string $newName
     */
    public function changeName(string $newName): void
    {
        $this->name = $newName;
        $this->raiseEvent(UpdatedParcelServiceName::create($this->id, $newName));
    }
    
    
    /**
     *
     */
    public function setAsDefault(): void
    {
        $this->isDefault = true;
        $this->raiseEvent(SetParcelServiceAsDefault::create($this->id));
    }
    
    
    /**
     * @param ParcelServiceDescriptions $newDescriptions
     */
    public function changeDescriptions(ParcelServiceDescriptions $newDescriptions): void
    {
        $this->descriptions = $newDescriptions;
        $this->raiseEvent(UpdatedParcelServiceDescriptions::create($this->id, $newDescriptions));
    }
    
    
    /**
     * @return array
     */
    public function toArray(): array
    {
        return [
            'id'           => $this->id->value(),
            'name'         => $this->name,
            'isDefault'    => $this->isDefault,
            'descriptions' => $this->descriptions->toArray(),
        ];
    }
    
}