<?php
/* --------------------------------------------------------------
   RedirectRulesAdminController.php 2020-08-12
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/
declare(strict_types=1);

namespace Gambio\Admin\Modules\RedirectRules;

use Gambio\Admin\Application\Http\Controller\JSEngineController;
use Gambio\Admin\Modules\RedirectRules\Exceptions\RedirectRuleSyntaxErrorException;
use Gambio\Admin\Modules\RedirectRules\Factories\RedirectServiceFactory;
use Gambio\Core\Configuration\ConfigurationService;
use Gambio\Core\Configuration\Models\Write\Configuration;
use Slim\Http\Response;
use Slim\Http\ServerRequest;

class RedirectRulesAdminController extends JSEngineController
{
    protected $service;
    protected $configurationService;
    
    
    public function __construct(RedirectServiceFactory $serviceFactory, ConfigurationService $configurationService)
    {
        $this->service              = $serviceFactory->service();
        $this->configurationService = $configurationService;
    }
    
    
    public function showOverview(ServerRequest $request, Response $response): Response
    {
        $this->addJsSectionTranslation('redirectrules');
        $template = $this->render($this->translate('module_title', 'redirectrules'),
                                  __DIR__ . '/ui/configuration.html');
        
        return $response->write($template);
    }
    
    
    public function getConfig(ServerRequest $request, Response $response): Response
    {
        $maxAge = 3600;
        if ($this->configurationService->has('redirect/maxAge')) {
            $maxAge = (int)$this->configurationService->find('redirect/maxAge')->value();
        }
        
        return $response->withJson(['maxAge' => $maxAge,]);
    }
    
    
    public function setConfig(ServerRequest $request, Response $response): Response
    {
        $data = $request->getParsedBody();
        $_SESSION['coo_page_token']->is_valid($data['page_token'] ?? '');
        
        if (isset($data['maxAge'])) {
            $this->configurationService->save(Configuration::create('redirect/maxAge', (string)(int)$data['maxAge']));
        }
        
        return $response->withJson(['result' => 'OK',]);
    }
    
    
    public function getRules(ServerRequest $request, Response $response): Response
    {
        $rules = $this->service->getRedirectRules();
        
        return $response->withJson(['rules' => $rules->toArray()]);
    }
    
    
    public function addRule(ServerRequest $request, Response $response): Response
    {
        $data = $request->getParsedBody();
        $_SESSION['coo_page_token']->is_valid($data['page_token'] ?? '');
        $newRuleData = $data['newRule'];
        if (empty($newRuleData['query']) && strpos($newRuleData['urlPath'], '?') !== false) {
            [$urlPath, $query] = explode('?', $newRuleData['urlPath'], 2);
        } else {
            $urlPath = $newRuleData['urlPath'];
            $query   = $newRuleData['query'];
        }
        $status = isset($newRuleData['status']) ? $newRuleData['status'] === 'true' : true;
        $maxAge = isset($newRuleData['maxAge']) ? (int)$newRuleData['maxAge'] : 3600;
        
        $responseData = [
            'message' => '',
        ];
        
        try {
            $newRule      = $this->service->makeRedirectRule(0,
                                                             $urlPath,
                                                             $query,
                                                             $newRuleData['queryMatchMode'],
                                                             (int)$newRuleData['responseCode'],
                                                             $newRuleData['target'],
                                                             $newRuleData['queryProcessing'],
                                                             $status);
            $insertedRule = $this->service->addRedirectRule($newRule);
        } catch (RedirectRuleSyntaxErrorException $e) {
            $responseData['message'] = $e->getUserMessage() ? : 'syntax_error';
        }
        
        return $response->withJson($responseData);
    }
    
    
    public function deleteRule(ServerRequest $request, Response $response): Response
    {
        $data = $request->getParsedBody();
        $_SESSION['coo_page_token']->is_valid($data['page_token'] ?? '');
        $ruleId = (int)$data['ruleId'];
        $this->service->deleteRedirectRule($ruleId);
        
        return $response->withJson(['message' => '']);
    }
    
    
    public function enableRule(ServerRequest $request, Response $response): Response
    {
        $data = $request->getParsedBody();
        $_SESSION['coo_page_token']->is_valid($data['page_token'] ?? '');
        $ruleId = (int)$data['ruleId'];
        $this->service->enableRedirectRule($ruleId);
        
        return $response->withJson(['message' => '']);
    }
    
    
    public function disableRule(ServerRequest $request, Response $response): Response
    {
        $data = $request->getParsedBody();
        $_SESSION['coo_page_token']->is_valid($data['page_token'] ?? '');
        $ruleId = (int)$data['ruleId'];
        $this->service->disableRedirectRule($ruleId);
        
        return $response->withJson(['message' => '']);
    }
    
    
    public function updateRule(ServerRequest $request, Response $response): Response
    {
        $data = $request->getParsedBody();
        $_SESSION['coo_page_token']->is_valid($data['page_token'] ?? '');
        $ruleData = $data['ruleData'];
        
        if (empty($ruleData['query']) && strpos($ruleData['urlPath'], '?') !== false) {
            [$urlPath, $query] = explode('?', $ruleData['urlPath'], 2);
        } else {
            $urlPath = $ruleData['urlPath'];
            $query   = $ruleData['query'];
        }
        $status = isset($ruleData['status']) ? $ruleData['status'] === 'true' : true;
        $maxAge = isset($ruleData['maxAge']) ? (int)$ruleData['maxAge'] : 3600;
        
        $responseMessage = '';
        try {
            $redirectRule = $this->service->makeRedirectRule((int)$ruleData['id'],
                                                             $urlPath,
                                                             $query,
                                                             $ruleData['queryMatchMode'],
                                                             (int)$ruleData['responseCode'],
                                                             $ruleData['target'],
                                                             $ruleData['queryProcessing'],
                                                             $status);
            $this->service->updateRedirectRule($redirectRule);
        } catch (RedirectRuleSyntaxErrorException $e) {
            $responseMessage = $e->getUserMessage() ? : 'Syntax error in redirect rule: ' . $e->getMessage();
        }
        
        return $response->withJson(['message' => $responseMessage]);
    }
}