<?php
/*--------------------------------------------------------------
  Timespan.php 2021-05-28
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2021 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  -------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\Statistics\Entities;

use DateInterval;
use DateTime;
use DateTimeInterface;
use Gambio\Admin\Modules\Statistics\Factories\Exceptions\InvalidDateTimespanException;

/**
 * Class Timespan
 *
 * @package Gambio\Admin\Modules\Statistics\Entities
 */
class Timespan
{
    /**
     * @var DateTime
     */
    protected $startDate;
    
    /**
     * @var DateTime
     */
    protected $endDate;
    
    /**
     * @var DateTime
     */
    protected $comparisonStartDate;
    
    /**
     * @var DateTime
     */
    protected $comparisonEndDate;
    
    /**
     * @var DateTime
     */
    protected $totalsStartDate;
    
    /**
     * @var DateTime
     */
    protected $totalsEndDate;
    
    /**
     * @var DateTime
     */
    protected $comparisonTotalsStartDate;
    
    /**
     * @var DateTime
     */
    protected $comparisonTotalsEndDate;
    
    /**
     * @var string
     */
    private $name;
    
    
    /**
     * Timespan constructor.
     *
     * @param DateTimeInterface      $startDate
     * @param DateTimeInterface      $endDate
     * @param string                 $name
     * @param DateTimeInterface|null $totalsStartDate
     * @param DateTimeInterface|null $totalsEndDate
     *
     * @throws InvalidDateTimespanException
     */
    public function __construct(
        DateTimeInterface $startDate,
        DateTimeInterface $endDate,
        string $name = '',
        ?DateTimeInterface $totalsStartDate = null,
        ?DateTimeInterface $totalsEndDate = null
    ) {
        if ($startDate > $endDate) {
            throw new InvalidDateTimespanException('The start date cannot be greater than the end date');
        }
        $this->startDate = $startDate;
        $this->endDate   = $endDate;
        
        switch ($name) {
            case 'thisweek':
            case 'last7days':
                $interval       = '7 days';
                $totalsInterval = '7 days';
                break;
            case 'last28days':
                $interval       = '28 days';
                $totalsInterval = '28 days';
                break;
            case 'thismonth':
                $interval       = '1 month';
                $totalsInterval = '1 month';
                break;
            case 'thisyear':
            case 'last12months':
                $interval       = '1 year';
                $totalsInterval = '1 year';
                break;
            case 'today':
            default:
                $interval       = '7 days';
                $totalsInterval = '1 day';
                break;
        }
        
        $this->comparisonStartDate = $startDate->sub(DateInterval::createFromDateString($interval));
        $this->comparisonEndDate   = $startDate->sub(DateInterval::createFromDateString('1 usec'));
        $this->name                = $name;
        
        $this->totalsStartDate           = $totalsStartDate ?? $startDate;
        $this->totalsEndDate             = $totalsEndDate ?? $endDate;
        $this->comparisonTotalsStartDate = $this->totalsStartDate->sub(DateInterval::createFromDateString($totalsInterval));
        $this->comparisonTotalsEndDate   = $this->totalsStartDate->sub(DateInterval::createFromDateString('1 usec'));
    }
    
    
    /**
     * @return DateTime
     */
    public function startDate(): DateTimeInterface
    {
        return $this->startDate;
    }
    
    
    /**
     * @return DateTime
     */
    public function endDate(): DateTimeInterface
    {
        return $this->endDate;
    }
    
    
    /**
     * @return DateTime
     */
    public function comparisonStartDate(): DateTimeInterface
    {
        return $this->comparisonStartDate;
    }
    
    
    /**
     * @return DateTime
     */
    public function comparisonEndDate(): DateTimeInterface
    {
        return $this->comparisonEndDate;
    }
    
    
    /**
     * @return string
     */
    public function name(): string
    {
        return $this->name;
    }
    
    
    /**
     * @return DateTime
     */
    public function totalsStartDate(): DateTimeInterface
    {
        return $this->totalsStartDate;
    }
    
    
    /**
     * @return DateTime
     */
    public function totalsEndDate(): DateTimeInterface
    {
        return $this->totalsEndDate;
    }
    
    
    /**
     * @return DateTime
     */
    public function comparisonTotalsEndDate(): DateTimeInterface
    {
        return $this->comparisonTotalsEndDate;
    }
    
    
    /**
     * @return DateTime
     */
    public function comparisonTotalsStartDate(): DateTimeInterface
    {
        return $this->comparisonTotalsStartDate;
    }
}
