<?php
/*--------------------------------------------------------------
  MysqlDataAccessObject.php 2021-05-28
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2021 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  -------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\Statistics\Repositories\Conversion\Reader;

use Doctrine\DBAL\Connection;
use Exception;
use Gambio\Admin\Modules\Statistics\Collections\StatisticValueCollection;
use Gambio\Admin\Modules\Statistics\Entities\StatisticValue;
use Gambio\Admin\Modules\Statistics\Entities\Timespan;
use Gambio\Admin\Modules\Statistics\ValueObjects\DateString;

/**
 * Class MysqlDataAccessObject
 * @package Gambio\Admin\Modules\Statistics\Repositories\Conversion\Reader
 */
class MysqlDataAccessObject implements ConversionReader
{
    /**
     * @var Connection
     */
    private $connection;
    
    /**
     * @var int[]
     */
    private $excludeOrderStatusIds = [99];
    
    /**
     * @var int
     */
    private const CONVERSION_RATE_PRECISION = 2;
    
    
    /**
     * MysqlDataAccessObject constructor.
     *
     * @param Connection $connection
     */
    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
    }
    
    
    /**
     * @inheritDoc
     * @throws Exception
     */
    public function fromTimespan(Timespan $timespan): StatisticValueCollection
    {
        $queryBuilder = $this->connection->createQueryBuilder();
        $orders       = $queryBuilder->select('DATE(date_purchased) as purchased_date, COUNT(*) as purchases')
            ->from('orders')
            ->where('orders_status NOT IN (' . implode(',', $this->excludeOrderStatusIds) . ')')
            ->andWhere("date_purchased BETWEEN '{$timespan->startDate()->format('Y-m-d H:i:s')}' AND '{$timespan->endDate()->format('Y-m-d H:i:s')}'")
            ->groupBy('DATE(date_purchased)')
            ->orderBy('DATE(date_purchased)')
            ->execute()
            ->fetchAll();
        
        $queryBuilder = $this->connection->createQueryBuilder();
        $visitors     = $queryBuilder->select('DATE(gm_counter_date) as visitor_date, gm_counter_visits_total as visitors')
            ->from('gm_counter_visits')
            ->andWhere("gm_counter_date BETWEEN '{$timespan->startDate()->format('Y-m-d H:i:s')}' AND '{$timespan->endDate()->format('Y-m-d H:i:s')}'")
            ->orderBy('DATE(gm_counter_date)')
            ->execute()
            ->fetchAll();
        
        return $this->createStatisticValueCollectionFromArray($orders, $visitors);
    }
    
    
    /**
     * @param string[] $orders
     *
     * @param string[] $visitors
     *
     * @return StatisticValueCollection
     * @throws Exception
     */
    private function createStatisticValueCollectionFromArray(array $orders, array $visitors): StatisticValueCollection
    {
        $result = new StatisticValueCollection;
        
        if (count($orders)) {
            
            foreach ($orders as ['purchased_date' => $purchasedDate, 'purchases' => $purchases]) {
                
                foreach ($visitors as ['visitor_date' => $visitorDate, 'visitors' => $visitorCounter]) {
                    
                    if ($visitorDate === $purchasedDate) {
                        
                        $date = $this->createDateString($purchasedDate);
                        $rate = round((int)$purchases / (int)$visitorCounter * 100,
                                      self::CONVERSION_RATE_PRECISION);
                        $result[] = $this->createStatisticValue($date, (float)$rate);
                        break;
                    }
                }
            }
        }
        
        return $result;
    }
    
    
    /**
     * @param string $date
     *
     * @return DateString
     * @throws Exception
     */
    private function createDateString(string $date): DateString
    {
        return new DateString($date);
    }
    
    
    /**
     * @param DateString $date
     * @param float      $value
     *
     * @return StatisticValue
     */
    private function createStatisticValue(DateString $date, float $value): StatisticValue
    {
        return new StatisticValue($date, $value);
    }
}