<?php
/* --------------------------------------------------------------
  MysqlDataAccessObject.php 2021-05-28
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2021 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\Statistics\Repositories\Orders\Readers;

use Doctrine\DBAL\Connection;
use Gambio\Admin\Modules\Statistics\Collections\StatisticValueCollection;
use Gambio\Admin\Modules\Statistics\Entities\StatisticValue;
use Gambio\Admin\Modules\Statistics\Entities\Timespan;
use Gambio\Admin\Modules\Statistics\ValueObjects\DateString;

/**
 * Class MysqlDataAccessObject
 * @package Gambio\Admin\Modules\Statistics\Repositories\Orders\Readers
 */
class MysqlDataAccessObject implements OrdersReader
{
    private const DATABASE_TABLE_NAME_ORDERS       = "orders";
    private const DATABASE_TABLE_KEY_ORDERS_STATUS = "orders_status";
    private const DATABASE_TABLE_KEY_ORDERS_DATE   = "date_purchased";
    
    /**
     * @var Connection
     */
    private $connection;
    
    /**
     * @var int[]
     */
    private $excludeOrderStatusIds = [99];
    
    
    /**
     * ParcelServiceReader constructor.
     *
     * @param Connection $connection
     */
    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
    }
    
    
    /**
     * @inheritDoc
     */
    public function getByTimespan(Timespan $timespan) : StatisticValueCollection
    {
        $startDate = $timespan->startDate()->format('Y-m-d H:i:s');
        $endDate = $timespan->endDate()->format('Y-m-d H:i:s');
        $ordersTableAlias = 'o';
        $purchaseDateKeyName = "{$ordersTableAlias}." . self::DATABASE_TABLE_KEY_ORDERS_DATE;
        $orderStatusKeyName = "{$ordersTableAlias}." . self::DATABASE_TABLE_KEY_ORDERS_STATUS;
        $selects = [
            "DATE({$purchaseDateKeyName}) AS orders_date",
            "COUNT({$purchaseDateKeyName}) AS orders_count",
        ];
        $queryBuilder         = $this->connection->createQueryBuilder();
        $queryResult = $queryBuilder->select($selects)
                                    ->from(self::DATABASE_TABLE_NAME_ORDERS, $ordersTableAlias)
                                    ->andWhere("{$purchaseDateKeyName} BETWEEN '{$startDate}' AND '{$endDate}'")
                                    ->andWhere($orderStatusKeyName . ' NOT IN (' . implode(',', $this->excludeOrderStatusIds) . ')')
                                    ->groupBy("DATE({$purchaseDateKeyName})")
                                    ->orderBy("DATE({$purchaseDateKeyName})")
                                    ->execute()
                                    ->fetchAll();
    
        return $this->createStatisticValueCollectionFromArray($queryResult);
    }
    
    
    /**
     * @param array $array
     *
     * @return StatisticValueCollection
     * @throws \Exception
     */
    private function createStatisticValueCollectionFromArray(array $array = []) : StatisticValueCollection
    {
        $result = new StatisticValueCollection;
        if (count($array)) {
            foreach ($array as ['orders_date' => $purchasedDate, 'orders_count' => $count]) {
                $result[] = new StatisticValue(
                    new DateString($purchasedDate), (float)$count
                );
            }
        }
        return $result;
    }
}