<?php
/*--------------------------------------------------------------
   MysqlDataAccessObject.php 2021-06-08
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2021 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
 -------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\Statistics\Repositories\Sales\Reader;

use Doctrine\DBAL\Connection;
use Gambio\Admin\Modules\Statistics\Collections\StatisticValueCollection;
use Gambio\Admin\Modules\Statistics\Entities\StatisticValue;
use Gambio\Admin\Modules\Statistics\Entities\Timespan;
use Gambio\Admin\Modules\Statistics\ValueObjects\DateString;

/**
 * Class MysqlDataAccessObject
 * @package Gambio\Admin\Modules\Statistics\Repositories\Sales\Reader
 */
class MysqlDataAccessObject implements SalesReader
{
    /**
     * @var Connection
     */
    private $connection;
    
    /**
     * @var int[]
     */
    private $excludeOrderStatusIds = [99];
    
    
    /**
     * MysqlDataAccessObject constructor.
     *
     * @param Connection $connection
     */
    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
    }
    
    
    /**
     * @inheritDoc
     */
    public function fromTimespan(Timespan $timespan): StatisticValueCollection
    {
        $queryBuilder = $this->connection->createQueryBuilder();
        $salesResult  = $queryBuilder->select('DATE(o.date_purchased) as purchased_date, SUM(ROUND(ot.value/o.currency_value, 2)) as sales')
            ->from('orders', 'o')
            ->from('orders_total', 'ot')
            ->where('o.orders_id=ot.orders_id')
            ->andWhere('o.orders_status NOT IN (' . implode(',', $this->excludeOrderStatusIds) . ')')
            ->andWhere('ot.class="ot_total"')
            ->andWhere("o.date_purchased BETWEEN '{$timespan->startDate()->format('Y-m-d H:i:s')}' AND '{$timespan->endDate()->format('Y-m-d H:i:s')}'")
            ->groupBy('DATE(o.date_purchased)')
            ->orderBy('DATE(o.date_purchased)')
            ->execute()
            ->fetchAll();
        
        $taxesResult = $queryBuilder->select('DATE(o.date_purchased) as purchased_date, SUM(ROUND(ot.value/o.currency_value, 2)) as taxes')
            ->from('orders', 'o')
            ->from('orders_total', 'ot')
            ->where('o.orders_id=ot.orders_id')
            ->andWhere('o.orders_status NOT IN (' . implode(',', $this->excludeOrderStatusIds) . ')')
            ->andWhere('ot.class="ot_tax"')
            ->andWhere("o.date_purchased BETWEEN '{$timespan->startDate()->format('Y-m-d H:i:s')}' AND '{$timespan->endDate()->format('Y-m-d H:i:s')}'")
            ->groupBy('DATE(o.date_purchased)')
            ->orderBy('DATE(o.date_purchased)')
            ->execute()
            ->fetchAll();
        
        return $this->createStatisticValueCollectionFromArray($salesResult, $taxesResult);
    }
    
    
    /**
     * @param array $salesRrows
     * @param array $taxesRows
     *
     * @return StatisticValueCollection
     * @throws \Exception
     */
    private function createStatisticValueCollectionFromArray(
        array $salesRrows,
        array $taxesRows
    ): StatisticValueCollection {
        $result = new StatisticValueCollection;
        
        $taxes = [];
        foreach ($taxesRows as $row) {
            $taxes[$row['purchased_date']] = $row['taxes'];
        }
        
        if (count($salesRrows)) {
            
            foreach ($salesRrows as ['purchased_date' => $purchasedDate, 'sales' => $sales]) {
                
                $date = $this->createDateString($purchasedDate);
                $sum = isset($taxes[$purchasedDate]) ? (float)$sales - (float)$taxes[$purchasedDate] : (float)$sales;
                $result[] = $this->createStatisticValue($date, $sum);
            }
        }
        
        return $result;
    }
    
    
    /**
     * @param string $date
     *
     * @return DateString
     * @throws \Exception
     */
    private function createDateString(string $date): DateString
    {
        return new DateString($date);
    }
    
    
    /**
     * @param DateString $date
     * @param float      $value
     *
     * @return StatisticValue
     */
    private function createStatisticValue(DateString $date, float $value): StatisticValue
    {
        return new StatisticValue($date, $value);
    }
}