<?php
/*--------------------------------------------------------------
  StatisticsService.php 2021-06-07
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2021 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  -------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\Statistics;

use Gambio\Admin\Modules\Statistics\Collections\CategoryCollection;
use Gambio\Admin\Modules\Statistics\Collections\SummarizedDataCollection;
use Gambio\Admin\Modules\Statistics\Entities\SummarizableTimespan;
use Gambio\Admin\Modules\Statistics\Exceptions\NoValidUserPreferredCategoryException;
use Gambio\Admin\Modules\Statistics\Exceptions\NoValidUserPreferredPeriodException;
use Gambio\Admin\Modules\Statistics\Factories\SummarizableTimespanFactory;
use Gambio\Admin\Modules\Statistics\Repositories\StatisticsDataProvider;
use Gambio\Admin\Modules\Statistics\Storage\StatisticsStorage;
use Gambio\Admin\Modules\Statistics\ValueObjects\Category;
use Gambio\Admin\Modules\Statistics\ValueObjects\StatisticsResult;
use Gambio\Admin\Modules\Statistics\ValueObjects\UserPreferredCategory;
use Gambio\Admin\Modules\Statistics\ValueObjects\UserPreferredPeriod;

/**
 * Class StatisticsService
 *
 * @package Gambio\Admin\Modules\Statistics
 */
class StatisticsService implements StatisticsServiceInterface
{
    /**
     * @var StatisticsDataProvider[]
     */
    private $dataProviders;
    
    /**
     * @var SummarizableTimespanFactory
     */
    private $factory;
    
    /**
     * @var StatisticsStorage
     */
    private $storage;
    
    
    /**
     * StatisticsService constructor.
     *
     * @param SummarizableTimespanFactory $factory
     * @param StatisticsStorage           $storage
     * @param StatisticsDataProvider[]    $dataProviders
     */
    public function __construct(
        SummarizableTimespanFactory $factory,
        StatisticsStorage $storage,
        StatisticsDataProvider ...$dataProviders
    ) {
        $this->dataProviders = $dataProviders;
        $this->factory       = $factory;
        $this->storage       = $storage;
    }
    
    
    /**
     * @inheritDoc
     */
    public function statistics(): StatisticsResult
    {
        
        $timespanDataFilter = $this->factory->createForTheWholePeriod();
        $resultDataArray    = [];
        $categoriesArray    = [];
        foreach ($this->dataProviders as $provider) {
            $data              = $provider->fromTimespan($timespanDataFilter);
            $categoriesArray[] = new Category($data->name(),
                                              $data->title(),
                                              $data->unit(),
                                              $data->style(),
                                              $data->minimumFractionDigits());
            $resultDataArray[] = $data;
        }
        
        $statistics            = [];
        $timespanPresentations = $this->factory->createAll();
        /** @var SummarizableTimespan $timespan */
        foreach ($timespanPresentations as $timespan) {
            $statistics[] = $timespan->summarize(...$resultDataArray);
        }
        
        return new StatisticsResult(new SummarizedDataCollection($statistics),
                                    new CategoryCollection($categoriesArray));
    }
    
    
    /**
     * @inheritDoc
     */
    public function updateUserPreferredPeriod(string $period): void
    {
        try {
            $userPreferredPeriod = new UserPreferredPeriod($period);
            $this->storage->setPreferredPeriod($userPreferredPeriod);
        } catch (NoValidUserPreferredPeriodException $e) {
            unset($e);
        }
    }
    
    
    /**
     * @inheritDoc
     */
    public function getUserPreferredPeriod(): ?UserPreferredPeriod
    {
        return $this->storage->getPreferredPeriod();
    }
    
    
    /**
     * @param string $category
     */
    public function updateUserPreferredCategory(string $category): void
    {
        try {
            $userPreferredCategory = new UserPreferredCategory($category);
            $this->storage->setPreferredCategory($userPreferredCategory);
        } catch (NoValidUserPreferredCategoryException $e) {
            unset($e);
        }
    }
    
    
    /**
     * @return UserPreferredCategory|null
     * @throws NoValidUserPreferredCategoryException
     */
    public function getUserPreferredCategory(): ?UserPreferredCategory
    {
        return $this->storage->getPreferredCategory();
    }
}