<?php
/*------------------------------------------------------------------------------
  StatisticsServiceProvider.php 2021-06-07
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2021 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  -----------------------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\Statistics;

use Doctrine\DBAL\Connection;
use Gambio\Admin\Modules\Statistics\Factories\SummarizableTimespanFactory;
use Gambio\Admin\Modules\Statistics\Factories\TimespanFactory;
use Gambio\Admin\Modules\Statistics\Repositories\Conversion\ConversionRateDataProvider;
use Gambio\Admin\Modules\Statistics\Repositories\Conversion\Factories\ConversionDataProviderResultFactory;
use Gambio\Admin\Modules\Statistics\Repositories\Conversion\Reader\ConversionReader;
use Gambio\Admin\Modules\Statistics\Repositories\Conversion\Reader\MysqlDataAccessObject;
use Gambio\Admin\Modules\Statistics\Repositories\Orders\Factories\OrdersDataProviderResultFactory;
use Gambio\Admin\Modules\Statistics\Repositories\Orders\OrdersDataProvider;
use Gambio\Admin\Modules\Statistics\Repositories\Orders\Readers\OrdersReader;
use Gambio\Admin\Modules\Statistics\Repositories\Sales\Factories\SalesDataProviderResultFactory;
use Gambio\Admin\Modules\Statistics\Repositories\Sales\Reader\SalesReader;
use Gambio\Admin\Modules\Statistics\Repositories\Sales\SalesDataProvider;
use Gambio\Admin\Modules\Statistics\Repositories\Visitors\Factories\VisitorsDataProviderResultFactory;
use Gambio\Admin\Modules\Statistics\Repositories\Visitors\Readers\BasicVisitorReader;
use Gambio\Admin\Modules\Statistics\Repositories\Visitors\Readers\VisitorsReader;
use Gambio\Admin\Modules\Statistics\Repositories\Visitors\VisitorsDataProvider;
use Gambio\Admin\Modules\Statistics\Storage\StatisticsStorage;
use Gambio\Admin\Modules\Statistics\Storage\StatisticsStorageImplementation;
use Gambio\Admin\Modules\Statistics\ValueObjects\FirstDayOfTheWeek;
use Gambio\Core\Application\DependencyInjection\AbstractServiceProvider;
use Gambio\Core\Configuration\Compatibility\ConfigurationStorageRepositoryBuilder;
use Gambio\Core\Configuration\Services\ConfigurationService;
use Gambio\Core\Language\TextManager;

/**
 * Class StatisticsServiceProvider
 *
 * @package Gambio\Admin\Modules\Statistics
 * @codeCoverageIgnore
 */
class StatisticsServiceProvider extends AbstractServiceProvider
{
    /**
     * @return string[]
     */
    public function provides(): array
    {
        return [StatisticsServiceInterface::class];
    }
    
    
    public function register(): void
    {
        $this->registerConversion();
        $this->registerOrders();
        $this->registerVisitors();
        $this->registerSales();
        
        $this->application->registerShared(TimespanFactory::class, SummarizableTimespanFactory::class)
            ->addArgument(TextManager::class);
        
        $this->application->registerShared(StatisticsStorage::class, StatisticsStorageImplementation::class)
            ->addArgument(ConfigurationStorageRepositoryBuilder::class);
        
        $this->application->registerShared(StatisticsServiceInterface::class,
                                           \Gambio\Admin\Modules\Statistics\StatisticsService::class)
            ->addArgument(TimespanFactory::class)
            ->addArgument(StatisticsStorage::class)
            ->addArgument(VisitorsDataProvider::class)
            ->addArgument(OrdersDataProvider::class)
            ->addArgument(ConversionRateDataProvider::class)
            ->addArgument(SalesDataProvider::class);
    }
    
    
    private function registerConversion(): void
    {
        $this->application->registerShared(ConversionRateDataProvider::class)
            ->addArgument(ConversionReader::class)
            ->addArgument(ConversionDataProviderResultFactory::class);
        
        $this->application->registerShared(ConversionReader::class, MysqlDataAccessObject::class)
            ->addArgument(Connection::class);
        
        $this->application->registerShared(ConversionDataProviderResultFactory::class,
            function () {
                return new ConversionDataProviderResultFactory($this->application->get(TextManager::class));
            });
    }
    
    
    private function registerOrders(): void
    {
        $this->application->registerShared(OrdersDataProvider::class)
            ->addArgument(OrdersReader::class)
            ->addArgument(OrdersDataProviderResultFactory::class);
        
        $this->application->registerShared(OrdersReader::class,
            function () {
                return new \Gambio\Admin\Modules\Statistics\Repositories\Orders\Readers\MysqlDataAccessObject($this->application->get(Connection::class));
            })->addArgument(Connection::class);
        
        $this->application->registerShared(OrdersDataProviderResultFactory::class,
            function () {
                return new OrdersDataProviderResultFactory($this->application->get(TextManager::class));
            });
    }
    
    
    private function registerVisitors(): void
    {
        
        $this->application->registerShared(VisitorsDataProvider::class)
            ->addArgument(VisitorsReader::class)
            ->addArgument(VisitorsDataProviderResultFactory::class);
        
        $this->application->registerShared(VisitorsReader::class, BasicVisitorReader::class)
            ->addArgument(Connection::class);
        
        $this->application->registerShared(VisitorsDataProviderResultFactory::class,
            function () {
                return new VisitorsDataProviderResultFactory($this->application->get(TextManager::class));
            });
    }
    
    
    private function registerSales(): void
    {
        $this->application->registerShared(SalesDataProvider::class)
            ->addArgument(SalesReader::class)
            ->addArgument(SalesDataProviderResultFactory::class);
        
        $this->application->registerShared(SalesReader::class,
                                           \Gambio\Admin\Modules\Statistics\Repositories\Sales\Reader\MysqlDataAccessObject::class)
            ->addArgument(Connection::class);
        
        $this->application->registerShared(SalesDataProviderResultFactory::class,
            function () {
                return new SalesDataProviderResultFactory($this->application->get(TextManager::class),
                                                          $this->application->get(ConfigurationService::class));
            });
    }
}