<?php
/* --------------------------------------------------------------
  StatisticsStorageImplementation.php 2021-06-07
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2021 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Admin\Modules\Statistics\Storage;

use Gambio\Admin\Modules\Statistics\Exceptions\NoValidUserPreferredCategoryException;
use Gambio\Admin\Modules\Statistics\Exceptions\NoValidUserPreferredPeriodException;
use Gambio\Admin\Modules\Statistics\ValueObjects\UserPreferredCategory;
use Gambio\Admin\Modules\Statistics\ValueObjects\UserPreferredPeriod;
use Gambio\Core\Configuration\Compatibility\ConfigurationStorageRepository;
use Gambio\Core\Configuration\Compatibility\ConfigurationStorageRepositoryBuilder;

/**
 * Class StatisticsStorage
 * @package Gambio\Admin\Modules\Statistics\Storage
 */
class StatisticsStorageImplementation implements StatisticsStorage
{
    /**
     * Configuration storage namespace
     */
    private const CONFIG_STORAGE_NAMESPACE = 'modules/gambio/statistics';
    
    /**
     * Preferred Category Key
     */
    private const PREFERRED_CATEGORY = 'PREFERRED_CATEGORY';
    
    /**
     * Preferred Period Key
     */
    private const PREFERRED_PERIOD = 'PREFERRED_PERIOD';
    
    /**
     * @var string
     */
    private $categoryFallbackValue = 'sales';
    
    /**
     * @var string
     */
    private $periodFallbackValue = 'thisweek';
    
    /**
     * @var ConfigurationStorageRepository
     */
    private $repository;
    
    
    /**
     * StatisticsStorage constructor.
     *
     * @param ConfigurationStorageRepositoryBuilder $repositoryBuilder
     */
    public function __construct(ConfigurationStorageRepositoryBuilder $repositoryBuilder)
    {
        $this->repository = $repositoryBuilder->build(static::CONFIG_STORAGE_NAMESPACE);
    }
    
    
    /**
     * @inheritDoc
     */
    public function setPreferredPeriod(UserPreferredPeriod $period): void
    {
        $this->repository->set(static::PREFERRED_PERIOD, $period->value());
    }
    
    
    /**
     * @inheritDoc
     */
    public function getPreferredPeriod(): UserPreferredPeriod
    {
        $value = $this->repository->get(static::PREFERRED_PERIOD);
        $value = $value === '' ? $this->periodFallbackValue : $value;
        
        try {
            return new UserPreferredPeriod($value);
        } catch (NoValidUserPreferredPeriodException $exception) {
            return new UserPreferredPeriod($this->periodFallbackValue);
        }
    }
    
    
    /**
     * @inheritDoc
     */
    public function setPreferredCategory(UserPreferredCategory $category): void
    {
        $this->repository->set(static::PREFERRED_CATEGORY, $category->value());
    }
    
    
    /**
     * @inheritDoc
     */
    public function getPreferredCategory(): UserPreferredCategory
    {
        $value = $this->repository->get(static::PREFERRED_CATEGORY);
        $value = $value === '' ? $this->categoryFallbackValue : $value;
        
        try {
            return new UserPreferredCategory($value);
        } catch (NoValidUserPreferredCategoryException $exception) {
            return new UserPreferredCategory($this->categoryFallbackValue);
        }
    }
}