<?php
/* --------------------------------------------------------------
   TrackingCodeCriteria.php 2020-09-01
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\TrackingCode\App\Data\Filter;

use Doctrine\DBAL\Query\QueryBuilder;
use Gambio\Admin\Modules\TrackingCode\Model\ValueObjects\TrackingCodeFilters as TrackingCodeFiltersInterface;
use Webmozart\Assert\Assert;

/**
 * Class TrackingCodeRestApiCriteria
 *
 * @package Gambio\Admin\Modules\TrackingCode\SqlCriteria
 */
class TrackingCodeFilters implements TrackingCodeFiltersInterface
{
    /**
     * @var array<string, array<string, string>>
     */
    private $filters = [];
    
    /**
     * @var string[]
     */
    private static $attributeColumnMapping = [
        'id'                         => 'orders_parcel_tracking_codes.orders_parcel_tracking_code_id',
        'orderId'                    => 'orders_parcel_tracking_codes.order_id',
        'code'                       => 'orders_parcel_tracking_codes.tracking_code',
        'parcelService.id'           => 'orders_parcel_tracking_codes.parcel_service_id',
        'parcelService.languageId'   => 'orders_parcel_tracking_codes.language_id',
        'parcelService.languageCode' => 'languages.code',
        'parcelService.name'         => 'orders_parcel_tracking_codes.parcel_service_name',
        'parcelService.url'          => 'orders_parcel_tracking_codes.url',
        'parcelService.comment'      => 'orders_parcel_tracking_codes.comment',
        'createdOn'                  => 'orders_parcel_tracking_codes.creation_date',
    ];
    
    /**
     * @var string[]
     */
    private static $allowedForLikeOperation = [
        'code',
        'parcelService.languageCode',
        'parcelService.name',
        'parcelService.url',
        'parcelService.comment',
    ];
    
    /**
     * @var string[]
     */
    private static $allowedForNumericOperation = [
        'id',
        'orderId',
        'parcelService.id',
        'parcelService.languageId',
        'createdOn',
    
    ];
    
    
    /**
     * TrackingCodeFilters constructor.
     */
    private function __construct()
    {
    }
    
    
    public static function create(array $filters): TrackingCodeFilters
    {
        Assert::isMap($filters, 'Provided filters need to be a map, mapping document attribute to filter.');
        Assert::allKeyExists($filters, 'value', 'Provided filters must contain a "value" key.');
        Assert::allKeyExists($filters, 'operation', 'Provided filters must contain a "operation" key.');
        
        $self = new self();
        foreach ($filters as $attribute => $filter) {
            $self->addFilter($attribute, $filter);
        }
        
        return $self;
    }
    
    
    /**
     * @param QueryBuilder $query
     */
    public function applyToQuery(QueryBuilder $query): void
    {
        foreach ($this->filters as $column => $filter) {
            $value     = $filter['value'];
            $operation = $filter['operation'];
            $parameter = ':criteria_filter_' . str_replace('.', '_', $column);
            
            if ($operation === 'like') {
                $value      = str_replace(['%', '*'], ['\%', '%'], $value);
                $filterExpr = $query->expr()->like($column, $parameter);
            } elseif ($operation === 'gt') {
                $filterExpr = $query->expr()->gt($column, $parameter);
            } elseif ($operation === 'gte') {
                $filterExpr = $query->expr()->gte($column, $parameter);
            } elseif ($operation === 'lt') {
                $filterExpr = $query->expr()->lt($column, $parameter);
            } elseif ($operation === 'lte') {
                $filterExpr = $query->expr()->lte($column, $parameter);
            } else {
                $filterExpr = $query->expr()->eq($column, $parameter);
            }
            
            $query->andWhere($filterExpr)->setParameter($parameter, $value);
        }
    }
    
    
    /**
     * @param string $attribute
     * @param array  $filter
     */
    private function addFilter(string $attribute, array $filter): void
    {
        $operations = ['like', 'eq', 'gt', 'gte', 'lt', 'lte'];
        Assert::oneOf($filter['operation'],
                      $operations,
                      'Provided operation needs to be one of: ' . implode(', ', $operations) . '; Got: '
                      . $filter['operation']);
        
        if ($filter['operation'] === 'like') {
            Assert::oneOf($attribute,
                          self::$allowedForLikeOperation,
                          'Provided attribute is not allowed for "like" operation.');
        } elseif (in_array($filter['operation'], ['gt', 'gte', 'lt', 'lte'])) {
            Assert::oneOf($attribute,
                          self::$allowedForNumericOperation,
                          'Provided attribute is not allowed for a numeric operation.');
        }
        
        $this->filters[self::$attributeColumnMapping[$attribute]] = $filter;
    }
}