<?php
/* --------------------------------------------------------------
   TrackingCodeRepository.php 2020-09-01
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2020 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\TrackingCode\App\Data;

use Gambio\Admin\Modules\TrackingCode\Model\Collections\TrackingCodeIds;
use Gambio\Admin\Modules\TrackingCode\Model\Collections\TrackingCodes;
use Gambio\Admin\Modules\TrackingCode\Model\Events\CreatedTrackingCode;
use Gambio\Admin\Modules\TrackingCode\Model\Events\DeletedTrackingCode;
use Gambio\Admin\Modules\TrackingCode\Model\TrackingCode;
use Gambio\Admin\Modules\TrackingCode\Model\ValueObjects\OrderId;
use Gambio\Admin\Modules\TrackingCode\Model\ValueObjects\ParcelServiceDetails;
use Gambio\Admin\Modules\TrackingCode\Model\ValueObjects\TrackingCodeFilters;
use Gambio\Admin\Modules\TrackingCode\Model\ValueObjects\TrackingCodeId;
use Gambio\Admin\Modules\TrackingCode\Model\ValueObjects\TrackingCodePagination;
use Gambio\Admin\Modules\TrackingCode\Model\ValueObjects\TrackingCodeSorting;
use Gambio\Admin\Modules\TrackingCode\Services\TrackingCodeRepository as TrackingCodeRepositoryInterface;
use Gambio\Core\Event\Abstracts\AbstractEventDispatchingRepository;
use InvalidArgumentException;
use Psr\EventDispatcher\EventDispatcherInterface;
use Webmozart\Assert\Assert;

/**
 * Class TrackingCodeRepository
 *
 * @package Gambio\Admin\Modules\TrackingCode\App\Data
 */
class TrackingCodeRepository extends AbstractEventDispatchingRepository implements TrackingCodeRepositoryInterface
{
    /**
     * @var TrackingCodeMapper
     */
    private $mapper;
    
    /**
     * @var TrackingCodeReader
     */
    private $reader;
    
    /**
     * @var TrackingCodeWriter
     */
    private $writer;
    
    
    /**
     * TrackingCodeRepository constructor.
     *
     * @param TrackingCodeMapper       $mapper
     * @param TrackingCodeReader       $reader
     * @param TrackingCodeWriter       $writer
     * @param EventDispatcherInterface $eventDispatcher
     */
    public function __construct(
        TrackingCodeMapper $mapper,
        TrackingCodeReader $reader,
        TrackingCodeWriter $writer,
        EventDispatcherInterface $eventDispatcher
    ) {
        $this->mapper = $mapper;
        $this->reader = $reader;
        $this->writer = $writer;
        
        $this->setEventDispatcher($eventDispatcher);
    }
    
    
    /**
     * @inheritDoc
     */
    public function filterTrackingCodes(
        TrackingCodeFilters $filters,
        TrackingCodeSorting $sorting,
        TrackingCodePagination $pagination
    ): TrackingCodes {
        $trackingCodesData = $this->reader->getFilteredTrackingCodesData($filters, $sorting, $pagination);
        
        return $this->mapper->mapTrackingCodes($trackingCodesData);
    }
    
    
    /**
     * @inheritDoc
     */
    public function getTrackingCodesTotalCount(TrackingCodeFilters $criteria): int
    {
        return $this->reader->getTrackingCodesTotalCount($criteria);
    }
    
    
    /**
     * @inheritDoc
     */
    public function getAllTrackingCodes(): TrackingCodes
    {
        $trackingCodesData = $this->reader->getAllTrackingCodesData();
        
        return $this->mapper->mapTrackingCodes($trackingCodesData);
    }
    
    
    /**
     * @inheritDoc
     */
    public function getTrackingCodeById(TrackingCodeId $id): TrackingCode
    {
        $trackingCodeData = $this->reader->getTrackingCodeDataById($id);
        
        return $this->mapper->mapTrackingCode($trackingCodeData);
    }
    
    
    /**
     * @inheritDoc
     */
    public function createTrackingCode(
        OrderId $orderId,
        string $code,
        ParcelServiceDetails $parcelServiceDetails
    ): TrackingCodeId {
        $idValue = $this->writer->createTrackingCode($orderId, $code, $parcelServiceDetails);
        $id      = $this->mapper->mapTrackingCodeId($idValue);
        
        $this->dispatchEvent(CreatedTrackingCode::create($id));
        
        return $id;
    }
    
    
    /**
     * @inheritDoc
     */
    public function createMultipleTrackingCodes(
        array $orderIds,
        array $codes,
        array $parcelServiceDetails
    ): TrackingCodeIds {
        Assert::allIsInstanceOf($orderIds,
                                OrderId::class,
                                'All order IDs need to implement the "' . OrderId::class . '" interface.');
        Assert::allIsInstanceOf($parcelServiceDetails,
                                ParcelServiceDetails::class,
                                'All parcel service details need to implement the "' . ParcelServiceDetails::class
                                . '" interface.');
        if (count($orderIds) !== count($parcelServiceDetails) || count($codes) !== count($parcelServiceDetails)) {
            throw new InvalidArgumentException('The number of given order IDs and codes must match the number of parcel service details.');
        }
        
        $idValues = $this->writer->createMultipleTrackingCodes($orderIds, $codes, $parcelServiceDetails);
        
        $ids = $this->mapper->mapTrackingCodeIds(...$idValues);
        foreach ($ids as $id) {
            $this->dispatchEvent(CreatedTrackingCode::create($id));
        }
        
        return $ids;
    }
    
    
    /**
     * @inheritDoc
     */
    public function deleteTrackingCodes(TrackingCodeId ...$ids): void
    {
        $this->writer->deleteTrackingCodes(...$ids);
        foreach ($ids as $id) {
            $this->dispatchEvent(DeletedTrackingCode::create($id));
        }
    }
    
    
    /**
     * @inheritcDoc
     */
    public function getTrackingCodesByOrderId(OrderId $orderId): TrackingCodes
    {
        $trackingCodesData = $this->reader->getTrackingCodesDataByOrderId($orderId);
        
        return $this->mapper->mapTrackingCodes($trackingCodesData);
    }
}