<?php
/* --------------------------------------------------------------
   TrackingCodeWriter.php 2020-09-01
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2020 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\TrackingCode\App\Data;

use Doctrine\DBAL\Connection;
use Exception;
use Gambio\Admin\Modules\TrackingCode\Model\Exceptions\CreationOfTrackingCodesFailed;
use Gambio\Admin\Modules\TrackingCode\Model\Exceptions\DeletionOfTrackingCodesFailed;
use Gambio\Admin\Modules\TrackingCode\Model\ValueObjects\OrderId;
use Gambio\Admin\Modules\TrackingCode\Model\ValueObjects\ParcelServiceDetails;
use Gambio\Admin\Modules\TrackingCode\Model\ValueObjects\TrackingCodeId;
use Gambio\Core\Language\LanguageService;

/**
 * Class TrackingCodeWriter
 *
 * @package Gambio\Admin\Modules\TrackingCode\App\Data
 */
class TrackingCodeWriter
{
    /**
     * @var Connection
     */
    private $db;
    
    /**
     * @var LanguageService
     */
    private $languageService;
    
    
    /**
     * TrackingCodeWriter constructor.
     *
     * @param Connection      $db
     * @param LanguageService $languageService
     */
    public function __construct(Connection $db, LanguageService $languageService)
    {
        $this->db              = $db;
        $this->languageService = $languageService;
    }
    
    
    /**
     * @param OrderId              $orderId
     * @param string               $code
     * @param ParcelServiceDetails $parcelServiceDetails
     *
     * @return int
     *
     * @throws CreationOfTrackingCodesFailed
     */
    public function createTrackingCode(OrderId $orderId, string $code, ParcelServiceDetails $parcelServiceDetails): int
    {
        try {
            $language = $this->languageService->getLanguageByCode($parcelServiceDetails->languageCode());
            
            $this->db->createQueryBuilder()
                ->insert('orders_parcel_tracking_codes')
                ->setValue('order_id', ':orderId')
                ->setValue('tracking_code', ':code')
                ->setValue('language_id', ':languageId')
                ->setValue('parcel_service_id', ':parcelServiceId')
                ->setValue('parcel_service_name', ':parcelServiceName')
                ->setValue('url', ':parcelServiceUrl')
                ->setValue('comment', ':parcelServiceComment')
                ->setValue('creation_date', 'now()')
                ->setParameter('orderId', $orderId->value())
                ->setParameter('code', $code)
                ->setParameter('languageId', $language->id())
                ->setParameter('parcelServiceId', $parcelServiceDetails->parcelServiceId())
                ->setParameter('parcelServiceName', $parcelServiceDetails->name())
                ->setParameter('parcelServiceUrl', $parcelServiceDetails->url())
                ->setParameter('parcelServiceComment', $parcelServiceDetails->comment())
                ->execute();
            
            return (int)$this->db->lastInsertId();
        } catch (Exception $exception) {
            throw CreationOfTrackingCodesFailed::becauseOfException($exception);
        }
    }
    
    
    /**
     * @param OrderId[]              $orderIds
     * @param string[]               $codes
     * @param ParcelServiceDetails[] $parcelServiceDetails
     *
     * @return int[]
     *
     * @throws CreationOfTrackingCodesFailed
     */
    public function createMultipleTrackingCodes(array $orderIds, array $codes, array $parcelServiceDetails): array
    {
        try {
            $this->db->beginTransaction();
            $ids = [];
            foreach ($orderIds as $index => $orderId) {
                $ids[] = $this->createTrackingCode($orderId, $codes[$index], $parcelServiceDetails[$index]);
            }
            $this->db->commit();
        } catch (CreationOfTrackingCodesFailed $exception) {
            $this->db->rollBack();
            throw $exception;
        } catch (Exception $exception) {
            $this->db->rollBack();
            throw CreationOfTrackingCodesFailed::becauseOfException($exception);
        }
        
        return $ids;
    }
    
    
    /**
     * @param TrackingCodeId $id
     */
    private function deleteTrackingCode(TrackingCodeId $id): void
    {
        $this->db->createQueryBuilder()
            ->delete('orders_parcel_tracking_codes')
            ->where('orders_parcel_tracking_code_id = :id')
            ->setParameter('id', $id->value())
            ->execute();
    }
    
    
    /**
     * @param TrackingCodeId ...$ids
     *
     * @throws DeletionOfTrackingCodesFailed
     */
    public function deleteTrackingCodes(TrackingCodeId ...$ids): void
    {
        try {
            $this->db->beginTransaction();
            foreach ($ids as $id) {
                $this->deleteTrackingCode($id);
            }
            $this->db->commit();
        } catch (Exception $exception) {
            $this->db->rollBack();
            throw DeletionOfTrackingCodesFailed::becauseOfException($exception);
        }
    }
}