<?php
/* --------------------------------------------------------------
 UserConfigurationController.php 2020-06-11
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Admin\Modules\UserConfiguration;

use Gambio\Admin\Modules\UserConfiguration\Interfaces\CurrentUserConfigurationService;
use Gambio\Admin\Modules\UserConfiguration\Model\UserConfiguration;
use Gambio\Admin\Modules\UserConfiguration\Model\UserConfigurationKey;
use Slim\Http\Response;
use Slim\Http\ServerRequest;
use Throwable;

/**
 * Class UserConfigurationController
 * @package Gambio\Admin\Modules\UserConfiguration
 */
class UserConfigurationController
{
    /**
     * @var CurrentUserConfigurationService
     */
    private $configurationService;
    
    
    /**
     * UserConfigurationController constructor.
     *
     * @param CurrentUserConfigurationService $configurationService
     */
    public function __construct(CurrentUserConfigurationService $configurationService)
    {
        $this->configurationService = $configurationService;
    }
    
    
    /**
     * Returns a user configuration value for currently logged in user.
     *
     * @param ServerRequest $request
     * @param Response      $response
     *
     * @return Response
     */
    public function get(ServerRequest $request, Response $response): Response
    {
        $rng = md5(time());
        
        $key   = $request->getQueryParam('key', 'key');
        $value = $this->configurationService->getValue($key, $rng);
        
        $success = $value !== $rng;
        
        return $response->withJson(['success' => $success, 'value' => $success ? $value : null]);
    }
    
    
    /**
     * Saves a user configuration for the currently logged in user.
     *
     * @param ServerRequest $request
     * @param Response      $response
     *
     * @return Response
     */
    public function save(ServerRequest $request, Response $response): Response
    {
        try {
            $key   = $request->getParsedBodyParam('key');
            $value = $request->getParsedBodyParam('value');
            
            $configKey     = UserConfigurationKey::create($key);
            $configuration = UserConfiguration::create($configKey, $value);
            $this->configurationService->storeConfiguration($configuration);
            
            $data = ['success' => true];
        } catch (Throwable $e) {
            $data = ['success' => false, 'message' => $e->getMessage()];
        }
        
        return $response->withJson($data);
    }
}