<?php
/* --------------------------------------------------------------
   WithdrawalCriteria.php 2020-08-24
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\Withdrawal\App\Data\Filter;

use Doctrine\DBAL\Query\QueryBuilder;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\WithdrawalFilters as WithdrawalFiltersInterface;
use Webmozart\Assert\Assert;

/**
 * Class WithdrawalRestApiCriteria
 *
 * @package Gambio\Admin\Modules\Withdrawal\SqlCriteria
 */
class WithdrawalFilters implements WithdrawalFiltersInterface
{
    /**
     * @var array<string, array<string, string>>
     */
    private $filters = [];
    
    /**
     * @var string[]
     */
    private static $attributeColumnMapping = [
        'id'                        => 'withdrawal_id',
        'order.id'                  => 'order_id',
        'order.creationDate'        => 'order_date',
        'order.deliveryDate'        => 'delivery_date',
        'customer.id'               => 'customer_id',
        'customer.gender'           => 'customer_gender',
        'customer.firstName'        => 'customer_firstname',
        'customer.lastName'         => 'customer_lastname',
        'customer.address.street'   => 'customer_street_address',
        'customer.address.postcode' => 'customer_postcode',
        'customer.address.city'     => 'customer_city',
        'customer.address.country'  => 'customer_country',
        'customer.email'            => 'customer_email',
        'date'                      => 'withdrawal_date',
        'content'                   => 'withdrawal_content',
        'createdByAdmin'            => 'created_by_admin',
        'createdOn'                 => 'date_created',
    ];
    
    /**
     * @var string[]
     */
    private static $allowedForLikeOperation = [
        'customer.gender',
        'customer.firstName',
        'customer.lastName',
        'customer.address.street',
        'customer.address.postcode',
        'customer.address.city',
        'customer.address.country',
        'customer.email',
        'content',
    ];
    
    /**
     * @var string[]
     */
    private static $allowedForNumericOperation = [
        'id',
        'order.id',
        'order.creationDate',
        'order.deliveryDate',
        'customer.id',
        'date',
        'createdOn',
    ];
    
    
    /**
     * WithdrawalFilters constructor.
     */
    private function __construct()
    {
    }
    
    
    public static function create(array $filters): WithdrawalFilters
    {
        Assert::isMap($filters, 'Provided filters need to be a map, mapping document attribute to filter.');
        Assert::allKeyExists($filters, 'value', 'Provided filters must contain a "value" key.');
        Assert::allKeyExists($filters, 'operation', 'Provided filters must contain a "operation" key.');
        
        $self = new self();
        foreach ($filters as $attribute => $filter) {
            $self->addFilter($attribute, $filter);
        }
        
        return $self;
    }
    
    
    /**
     * @param QueryBuilder $query
     */
    public function applyToQuery(QueryBuilder $query): void
    {
        foreach ($this->filters as $column => $filter) {
            $value     = $filter['value'];
            $operation = $filter['operation'];
            $parameter = ':criteria_filter_' . str_replace('.', '_', $column);
            
            if ($operation === 'like') {
                $value      = str_replace(['%', '*'], ['\%', '%'], $value);
                $filterExpr = $query->expr()->like($column, $parameter);
            } elseif ($operation === 'gt') {
                $filterExpr = $query->expr()->gt($column, $parameter);
            } elseif ($operation === 'gte') {
                $filterExpr = $query->expr()->gte($column, $parameter);
            } elseif ($operation === 'lt') {
                $filterExpr = $query->expr()->lt($column, $parameter);
            } elseif ($operation === 'lte') {
                $filterExpr = $query->expr()->lte($column, $parameter);
            } else {
                $filterExpr = $query->expr()->eq($column, $parameter);
            }
            
            $query->andWhere($filterExpr)->setParameter($parameter, $value);
        }
    }
    
    
    /**
     * @param string $attribute
     * @param array  $filter
     */
    private function addFilter(string $attribute, array $filter): void
    {
        $operations = ['like', 'eq', 'gt', 'gte', 'lt', 'lte'];
        Assert::oneOf($filter['operation'],
                      $operations,
                      'Provided operation needs to be one of: ' . implode(', ', $operations) . '; Got: '
                      . $filter['operation']);
        
        if ($filter['operation'] === 'like') {
            Assert::oneOf($attribute,
                          self::$allowedForLikeOperation,
                          'Provided attribute is not allowed for "like" operation.');
        } elseif (in_array($filter['operation'], ['gt', 'gte', 'lt', 'lte'])) {
            Assert::oneOf($attribute,
                          self::$allowedForNumericOperation,
                          'Provided attribute is not allowed for a numeric operation.');
        }
        
        $this->filters[self::$attributeColumnMapping[$attribute]] = $filter;
    }
}