<?php
/* --------------------------------------------------------------
   WithdrawalRepository.php 2020-09-01
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2020 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\Withdrawal\App\Data;

use DateTime;
use Gambio\Admin\Modules\Withdrawal\Model\Collections\WithdrawalIds;
use Gambio\Admin\Modules\Withdrawal\Model\Collections\Withdrawals;
use Gambio\Admin\Modules\Withdrawal\Model\Events\CreatedWithdrawal;
use Gambio\Admin\Modules\Withdrawal\Model\Events\DeletedWithdrawal;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\CustomerDetails;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\OrderDetails;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\WithdrawalFilters;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\WithdrawalId;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\WithdrawalPagination;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\WithdrawalSorting;
use Gambio\Admin\Modules\Withdrawal\Model\Withdrawal;
use Gambio\Admin\Modules\Withdrawal\Services\WithdrawalRepository as WithdrawalRepositoryInterface;
use Gambio\Core\Event\Abstracts\AbstractEventDispatchingRepository;
use InvalidArgumentException;
use Psr\EventDispatcher\EventDispatcherInterface;
use Webmozart\Assert\Assert;

/**
 * Class WithdrawalRepository
 *
 * @package Gambio\Admin\Modules\Withdrawal\App\Data
 */
class WithdrawalRepository extends AbstractEventDispatchingRepository implements WithdrawalRepositoryInterface
{
    /**
     * @var WithdrawalMapper
     */
    private $mapper;
    
    /**
     * @var WithdrawalReader
     */
    private $reader;
    
    /**
     * @var WithdrawalWriter
     */
    private $writer;
    
    
    /**
     * WithdrawalRepository constructor.
     *
     * @param WithdrawalMapper         $mapper
     * @param WithdrawalReader         $reader
     * @param WithdrawalWriter         $writer
     * @param EventDispatcherInterface $eventDispatcher
     */
    public function __construct(
        WithdrawalMapper $mapper,
        WithdrawalReader $reader,
        WithdrawalWriter $writer,
        EventDispatcherInterface $eventDispatcher
    ) {
        $this->mapper = $mapper;
        $this->reader = $reader;
        $this->writer = $writer;
        
        $this->setEventDispatcher($eventDispatcher);
    }
    
    
    /**
     * @inheritDoc
     */
    public function filterWithdrawals(
        WithdrawalFilters $filters,
        WithdrawalSorting $sorting,
        WithdrawalPagination $pagination
    ): Withdrawals {
        $withdrawalsData = $this->reader->getFilteredWithdrawalsData($filters, $sorting, $pagination);
        
        return $this->mapper->mapWithdrawals($withdrawalsData);
    }
    
    
    /**
     * @inheritDoc
     */
    public function getWithdrawalsTotalCount(WithdrawalFilters $criteria): int
    {
        return $this->reader->getWithdrawalsTotalCount($criteria);
    }
    
    
    /**
     * @inheritDoc
     */
    public function getAllWithdrawals(): Withdrawals
    {
        $withdrawalsData = $this->reader->getAllWithdrawalsData();
        
        return $this->mapper->mapWithdrawals($withdrawalsData);
    }
    
    
    /**
     * @inheritDoc
     */
    public function getWithdrawalById(WithdrawalId $id): Withdrawal
    {
        $withdrawalData = $this->reader->getWithdrawalDataById($id);
        
        return $this->mapper->mapWithdrawal($withdrawalData);
    }
    
    
    /**
     * @inheritDoc
     */
    public function createWithdrawal(
        OrderDetails $order,
        CustomerDetails $customer,
        string $date = '',
        string $content = '',
        bool $createdByAdmin = true
    ): WithdrawalId {
        $dateObj = new DateTime($date);
        $idValue = $this->writer->createWithdrawal($order, $customer, $dateObj, $content, $createdByAdmin);
        $id      = $this->mapper->mapWithdrawalId($idValue);
        
        $this->dispatchEvent(CreatedWithdrawal::create($id));
        
        return $id;
    }
    
    
    /**
     * @inheritDoc
     */
    public function createMultipleWithdrawals(
        array $orders,
        array $customers,
        array $dates = [],
        array $contents = [],
        array $createdByAdminFlags = []
    ): WithdrawalIds {
        Assert::allIsInstanceOf($orders,
                                OrderDetails::class,
                                'All order details need to implement "' . OrderDetails::class . '" interface.');
        Assert::allIsInstanceOf($customers,
                                CustomerDetails::class,
                                'All customer details need to implement "' . CustomerDetails::class . '" interface.');
        Assert::isList($orders, 'Provided order details need to be a list.');
        Assert::isList($customers, 'Provided customer details need to be a list.');
        Assert::isList($dates, 'Provided withdrawal dates need to be a list.');
        Assert::isList($contents, 'Provided contents need to be a list.');
        Assert::isList($createdByAdminFlags, 'Provided created-by-admin flags need to be a list.');
        
        if (count($orders) !== count($customers)) {
            throw new InvalidArgumentException('Different number of order details and customer details provided.');
        }
        
        $dates = array_map(static function (string $date): DateTime {
            return new DateTime($date);
        },
            $dates);
        
        $idValues = $this->writer->createMultipleWithdrawals($orders,
                                                             $customers,
                                                             $dates,
                                                             $contents,
                                                             $createdByAdminFlags);
        
        $ids = $this->mapper->mapWithdrawalIds(...$idValues);
        foreach ($ids as $id) {
            $this->dispatchEvent(CreatedWithdrawal::create($id));
        }
        
        return $ids;
    }
    
    
    /**
     * @inheritDoc
     */
    public function storeWithdrawals(Withdrawal ...$withdrawals): void
    {
        $this->writer->updateWithdrawals(...$withdrawals);
        foreach ($withdrawals as $withdrawal) {
            $this->dispatchEntityEvents($withdrawal);
        }
    }
    
    
    /**
     * @inheritDoc
     */
    public function deleteWithdrawals(WithdrawalId ...$ids): void
    {
        $this->writer->deleteWithdrawals(...$ids);
        foreach ($ids as $id) {
            $this->dispatchEvent(DeletedWithdrawal::create($id));
        }
    }
}