<?php
/* --------------------------------------------------------------
   WithdrawalRepository.php 2020-09-01
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2020 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\Withdrawal\Services;

use Gambio\Admin\Modules\Withdrawal\Model\Collections\WithdrawalIds;
use Gambio\Admin\Modules\Withdrawal\Model\Collections\Withdrawals;
use Gambio\Admin\Modules\Withdrawal\Model\Exceptions\CreationOfWithdrawalFailed;
use Gambio\Admin\Modules\Withdrawal\Model\Exceptions\DeletionOfWithdrawalsFailed;
use Gambio\Admin\Modules\Withdrawal\Model\Exceptions\StorageOfWithdrawalsFailed;
use Gambio\Admin\Modules\Withdrawal\Model\Exceptions\WithdrawalNotFoundException;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\CustomerDetails;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\OrderDetails;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\WithdrawalFilters;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\WithdrawalId;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\WithdrawalPagination;
use Gambio\Admin\Modules\Withdrawal\Model\ValueObjects\WithdrawalSorting;
use Gambio\Admin\Modules\Withdrawal\Model\Withdrawal;

/**
 * Interface WithdrawalRepository
 *
 * @package Gambio\Admin\Modules\Withdrawal\Services
 */
interface WithdrawalRepository
{
    /**
     * @param WithdrawalFilters    $filters
     * @param WithdrawalSorting    $sorting
     * @param WithdrawalPagination $pagination
     *
     * @return Withdrawals
     */
    public function filterWithdrawals(
        WithdrawalFilters $filters,
        WithdrawalSorting $sorting,
        WithdrawalPagination $pagination
    ): Withdrawals;
    
    
    /**
     * @param WithdrawalFilters $criteria
     *
     * @return int
     */
    public function getWithdrawalsTotalCount(WithdrawalFilters $criteria): int;
    
    
    /**
     * @return Withdrawals
     */
    public function getAllWithdrawals(): Withdrawals;
    
    
    /**
     * @param WithdrawalId $id
     *
     * @return Withdrawal
     *
     * @throws WithdrawalNotFoundException
     */
    public function getWithdrawalById(WithdrawalId $id): Withdrawal;
    
    
    /**
     * @param OrderDetails    $order
     * @param CustomerDetails $customer
     * @param string          $date
     * @param string          $content
     * @param bool            $createdByAdmin
     *
     * @return WithdrawalId
     *
     * @throws CreationOfWithdrawalFailed
     */
    public function createWithdrawal(
        OrderDetails $order,
        CustomerDetails $customer,
        string $date = '',
        string $content = '',
        bool $createdByAdmin = true
    ): WithdrawalId;
    
    
    /**
     * @param OrderDetails[]    $orders
     * @param CustomerDetails[] $customers
     * @param string[]          $dates
     * @param string[]          $contents
     * @param bool[]            $createdByAdminFlags
     *
     * @return WithdrawalIds
     *
     * @throws CreationOfWithdrawalFailed
     */
    public function createMultipleWithdrawals(
        array $orders,
        array $customers,
        array $dates = [],
        array $contents = [],
        array $createdByAdminFlags = []
    ): WithdrawalIds;
    
    
    /**
     * @param Withdrawal ...$withdrawals
     *
     * @throws StorageOfWithdrawalsFailed
     */
    public function storeWithdrawals(Withdrawal ...$withdrawals): void;
    
    
    /**
     * @param WithdrawalId ...$ids
     *
     * @throws DeletionOfWithdrawalsFailed
     */
    public function deleteWithdrawals(WithdrawalId ...$ids): void;
}